//-------1---------2---------3---------4---------5---------6
#include "gfxcodec.h"
#include <unistd.h>

GfxCodec::GfxCodec(BDirectory *directory, BFile* a_file)
	: addon_IDName(0),
	addon_IDAuthor(0),
	addon_IDNotice(0),
	addon_IDEncoder(0),
	addon_IDDecoder(0),
	addon_CanCreate(0),
	next(0)
{
	a_file->GetName(fBaseName);
	fBaseDirectory = *directory;
	
	fIdentifier = B_ERROR;
	fDecoder = B_ERROR;
	fEncoder = B_ERROR;
	next = 0;

	// Try to load in the identifier module based on 
	// the name
	fIdentifier = load_add_on(a_file);
	
	if (B_ERROR != fIdentifier)
	{
		fDecoder = fIdentifier;
		fEncoder = fIdentifier;
		
		long error;
		
		// We have successfully loaded the module,
		// now get pointers to some functions we expect to exist.
		error = get_image_symbol(fIdentifier, "rrasaddon_IDName__Fv", 2, &addon_IDName);
		error = get_image_symbol(fIdentifier, "rrasaddon_IDAuthor__Fv", 2, &addon_IDAuthor);
		error = get_image_symbol(fIdentifier, "rrasaddon_IDNotice__Fv", 2, &addon_IDNotice);
		error = get_image_symbol(fIdentifier, "rrasaddon_IDEncoder__Fv", 2, &addon_IDEncoder);
		error = get_image_symbol(fIdentifier, "rrasaddon_IDDecoder__Fv", 2, &addon_IDDecoder);
		error = get_image_symbol(fIdentifier, "CanCreateImage__FPvl", 2, &addon_CanCreate);
		error = get_image_symbol(fIdentifier, "CreateImage__FP5BFile", 2, &addon_CreateImage);
		
		
		//if (error != B_ERROR && addon_IDName)
		//	printf("IDName loaded: %s\n", addon_IDName());
			
	} 
}

//=============================================
// Destructor
//
// Unload images
//=============================================
GfxCodec::~GfxCodec()
{
	long error = 0;
		
	// Unload the identifier module
	if (fIdentifier != B_ERROR)			
		error = unload_add_on(fIdentifier);

	// Unload the Decoder Module
	if (fDecoder != B_ERROR)			
		error = unload_add_on(fDecoder);
		
	// Unload the Encoder Module
	if (fEncoder != B_ERROR)			
		error = unload_add_on(fEncoder);

}

bool
GfxCodec::IsValid()
{
	// This is a valid codec if the addon code has been 
	// loaded, and the Module name identifier function 
	// has been found
    return ((B_ERROR != fIdentifier) && (addon_IDName));
}

static void
PrintTable(BTable *aTable)
{
	long nFields = aTable->CountFields();
	char name[1024];
	char TYPE[5];
	field_key aKey;
	long type;
	long flags;
	
	for (int ctr = 0; ctr < nFields; ctr++)
	{
		if (aTable->GetFieldInfo(ctr,name,&aKey,&type,&flags))
		{
			memcpy(TYPE,&type,4);
			TYPE[4] = '\0';
			printf("Field: %s %s\n", name, TYPE);
		}
	}
}

//=============================================
// Method: Print
//
// Print information related to this codec
//=============================================
void
GfxCodec::Print()
{
	long error = 0;
	image_info info;
	long data_len;
	
	// Print Base Name of module
	printf("%s\n", fBaseName);
	
	// Print the identifier module
	if (fIdentifier != B_ERROR)
	{		
		error = get_image_info (fIdentifier, &info);

		printf(" Identifier: /%s\n", info.name);
		printf("   Module Name: %s\n", addon_IDName());
		printf("   Module Author: %s\n", addon_IDAuthor());
		printf("   Module Notice: %s\n", addon_IDNotice());
		printf("   Module Encoder: %s\n", addon_IDEncoder());
		printf("   Module Decoder: %s\n", addon_IDDecoder());
	} else
		printf(" %s - No Identifier loaded\n", fBaseName);
	
	// Print the Decoder Module
	if (fDecoder != B_ERROR)			
	{		
		error = get_image_info (fDecoder, &info);
		printf(" Decoder: %s\n", info.name);
	} else
		printf(" %s - No Decoder loaded\n", fBaseName);
		
	// Print the Encoder Module
	if (fEncoder != B_ERROR)			
	{		
		error = get_image_info (fEncoder, &info);
		printf(" Encoder: %s\n", info.name);
	} else
		printf(" %s - No Encoder loaded\n", fBaseName);
}


//=====================================================
// Method: CanCreateImage
//
// This method is responsible for reporting back whether
// or not this module is capable of creating an image
// given the amount of data fed in.
//=====================================================

float
GfxCodec::CanCreateImage(BFile *file)
{
	// Early exits for lack of resources
	if (!file)
		return 0;

	if (!addon_CanCreate)
		return 0.0;

	char data[128];
	long buffSize =128;
	long dataLen=128;
	file->Open(B_READ_ONLY);
	file->Seek(0,B_SEEK_TOP);
	dataLen = file->Read(data, buffSize);

	float confidence = 	addon_CanCreate(data, dataLen);
	printf(" %s: GfxCodec::CanCreateImage - confidence: %f\n", addon_IDName(), confidence);
	
	return confidence;
}

GfxImage *	
GfxCodec::CreateImage(BFile *file)
{
	GfxImage *newImage = 0;
	
	// Early return due to lack of resources
	if (!file || !addon_CreateImage)
	{
		printf("GfxCodec::CreateImage() - leaving early\n");
		return 0;
	}	
/*
	if (B_ERROR == fDecoder)
	{
	BFile a_file;
	int error;
	record_ref aref;
	char aPath[1024];

		// Assume the image decoder is not loaded yet, and
		// load it.
		//fBaseDirectory->GetName(path);
		//strcat(path,"/");
		//strcat(path,fIDDecoder);
		//error = get_ref_for_path(path, &aref);
		
		// Create file from path
		//a_file.SetRef(aref);
		//a_file.GetName(path);
		//printf("GfxCodec::CreateImage - %s\n", path);
		//fDecoder = load_add_on(&a_file);
	}
	
	// If we still have an error, then the decoder could not
	// be loaded and we should just give up and return.
	if (B_ERROR == fDecoder)
	{
		return 0;
	}
*/

	// Now that we have the decoder add-on, we should be
	// able to call the decode function and get an image
	// out.
	newImage = addon_CreateImage(file);
	
	return newImage;
}


void		
GfxCodec::WriteImage(GfxImage*, BFile *)
{
}

