
#include <StorageKit.h>
#include <stdio.h>
#include <unistd.h>

#include "rraster.h"
#include "gfxcodec.h"

GfxCodec *gCodecList = 0;
BDirectory gAddOnsDirectory;

//---------------------------------------------
// Functions of the external interface
//
//---------------------------------------------
void
SetAddOnsDirectory(record_ref aRef)
{
	char name_buf[1024] = {'\0'};
	
	gAddOnsDirectory.SetRef(aRef);
	gAddOnsDirectory.GetName(name_buf);
	
	//printf("SetAddOnsDirectory - files: %d  dir.GetName: %s\n", 
	//	gAddOnsDirectory.CountFiles(), name_buf);

	ReloadCodecs();
}

void
PrintCodecList()
{
	image_id tmpID;
	GfxCodec *tmpCodec;
	
	// Iterate through the current codec list
	tmpCodec = gCodecList;
	while (tmpCodec)
	{
		tmpCodec->Print();
		tmpCodec = tmpCodec->next;
	}
}

void	
ReloadCodecs()
{
	image_id tmpID;
	GfxCodec *tmpCodec;
	
	// Iterate through the current codec list
	while (gCodecList)
	{
		long error = 0;
		
		tmpCodec = gCodecList->next;
		
		delete gCodecList;
		gCodecList = tmpCodec;
	}
	
	// If we don't have an add-ons directory,
	// then just return immediately.
	// This is one way of wiping out the add-ons
	if (gAddOnsDirectory.Error() == B_ERROR)
	{
		printf("ReloadCodecs - early exit, add-ons directory not valid.\n");
		return;
	}
		
	// Now traverse the current directory looking
	// for new codecs to build.
	long index;
	BFile a_file;
	BDirectory a_dir;
	long file_count, dir_count;
	record_ref *ref_vector;
	char name_buf[B_FILE_NAME_LENGTH];   
   
	file_count = gAddOnsDirectory.CountFiles();
	ref_vector = new record_ref[file_count];

	gAddOnsDirectory.GetName(name_buf);
	//printf("ReloadCodecs - Directory: %s  Count: %d\n",
	//	name_buf, file_count);
		
	gAddOnsDirectory.GetFiles(0, file_count, ref_vector);
   
	for (index = 0; index < file_count; index++) 
	{
		GfxCodec *newCodec = 0;
		
		// For each of the files found in the directory
		// Try to make a GfxCodec out of it.
		if (a_file.SetRef(ref_vector[index]) < B_NO_ERROR)
            continue;
		
		newCodec = new GfxCodec(&gAddOnsDirectory, &a_file);
		
		// If we didn't load the codec, or if
		// it isn't valid, then don't add it to
		// the list.
		if (!newCodec || !newCodec->IsValid())
			continue;
		
		// Make the new codec the beginning of the list if
		// the list is currently blank.  Otherwise
		// add it to the end of the list.
		if (0 == gCodecList)
			gCodecList = newCodec;
		else
			tmpCodec->next = newCodec;
		tmpCodec = newCodec;
	}

	delete [] ref_vector;
}


//=========================================================
// Decoder functions
//=========================================================

//==========================================
// Function: FindImageDecoder
//
// Goes through the decoder list asking each
// of the decoders if they know how to decode
// the image.  The one that responds with the
// highest score wins.
//
// We don't set any threshold other than > 0.0
// because some applications might just want to
// read the information in the header and not
// actually decode and create the whole image.
//==========================================

static GfxCodec *
FindImageDecoder(BFile *file)
{
	GfxCodec *candidate = 0;
	GfxCodec *tmpCodec = gCodecList;
	float confidence = 0.0;
	
	// Early exit if there is no file
	if (!file)
		return 0;
		
	
	// Traverse the list
	while (tmpCodec && confidence < 1.0)
	{
		float newConfidence = 0.0;
		newConfidence = tmpCodec->CanCreateImage(file);
		if (newConfidence > confidence)
		{
			confidence = newConfidence;
			candidate = tmpCodec;
		}
		
		tmpCodec = tmpCodec->next;
		
	}

	if ((confidence > 0.0) && candidate)
		return candidate;
			
	return 0;
}

GfxImage * 
CreateRasterFromFile(BFile *file)
{
	GfxImage *newImage = NULL;
	GfxCodec *codec = 0;
	
	codec = FindImageDecoder(file);
	
	// Create an image using the codec
	if (codec)
	{
		newImage = codec->CreateImage(file);
	} else
	{
		printf(" CreateRasterImageBFile - no appropriate codec found\n");
	}
	
	
	return newImage;
}

GfxImage * CreateRasterImage(char *fname)
{
	GfxImage *newImage = 0;
	BFile *file = 0;
	record_ref aRef;
	
	get_ref_for_path(fname, &aRef);
	file = new BFile(aRef);
	
	newImage = CreateRasterFromFile(file);

	delete file;
	
	return newImage;
}


//=========================================================
// Encoder functions
//=========================================================

static int WriteRasterImageBFile(GfxImage *p, BFile *, int fmt, void *options)
{
	return -1;
}

int WriteRasterImage(GfxImage *p, char *fname, int fmt, void *options)
{
	// Create a BFile based on the fname
	// Call the other function using the BFile
	return -1;
}



