/*
**	BTSRasterApp
*/

#include "rraster.h"
#include "rrasterapp.h"
#include "rrasterwindow.h"
 
BList window_list;
sem_id window_sem = -1;

BTSRasterApp::BTSRasterApp()
	:BApplication('GRVW'),
	fHomeDirectory(0)
{	
	printf("BTSRasterApp::BTSRasterApp()\n");

	window_sem = create_sem(1, "rraster_window");

	fHomeDirectory = new BDirectory();
	AddonsDirectory(fHomeDirectory);
	SetAddOnsDirectory(fHomeDirectory->Record()->Ref());
	
	PrintCodecList();
}

void
BTSRasterApp::ReadyToRun()
{
	printf("BTSRasterApp::ReadyToRun()\n");
}


bool
BTSRasterApp::QuitRequested()
{		
	return TRUE;
}

//--------------------------------------------------------------------
void 
BTSRasterApp::ArgvReceived(int argc, char **argv)
{
	printf("BTSRasterApp::ArgvReceived\n");

	record_ref	ref;
	
	for (int i = 1; i < argc; i++) 
	{
		if (get_ref_for_path(argv[i], &ref) == B_NO_ERROR) 
		{
			OpenFile(ref);
		}
	}

	QuitIfEmpty();
}


//--------------------------------------------------------------------
void 
BTSRasterApp::RefsReceived(BMessage *a_message)
{
	long		count;
	ulong		type;

	printf("BTSRasterApp::RefsReceived\n");

	if (a_message->GetInfo("refs", &type, &count)) 
	{
		for (long i = count; i>= 0; i--) 
		{
			record_ref item = a_message->FindRef("refs", i);
			if (a_message->Error() == B_ERROR)
				continue;
				
			if ((item.database >= 0) && (item.record >= 0))
			{
				if (does_ref_conform(item, "File"))
				{
					OpenFile(item);
				}
			}
		}
	}

	QuitIfEmpty();
}

void
BTSRasterApp::OpenFile(record_ref aRef)
{
	BFile * aFile = new BFile(aRef);
	GfxImage *newImage = CreateRasterFromFile(aFile);
	
	// Try to open the file
	// 
	if (newImage)
	{
		fContentWindow = new BTSRasterWindow("RRaster!", newImage);
		acquire_sem(window_sem);
		window_list.AddItem(fContentWindow);
		release_sem(window_sem);

		fContentWindow->Show();
	} else
	{
		BAlert *alert = 0;
		alert = new BAlert("", "BTSRasterApp::OpenFile() failed to create an image.", "Cancel");
		alert->Go();
		
		//be_app->PostMessage(B_QUIT_REQUESTED);
	}

	delete aFile;
}


void	
BTSRasterApp::MessageReceived(BMessage *msg)
{
	long	numLeft;
	BObject *aObject;
	
	switch (msg->what)
	{
		case RAS_REMOVE_WINDOW:
			aObject = msg->FindObject("window");
			if (aObject)
			{
				acquire_sem(window_sem);
				window_list.RemoveItem(aObject);
				release_sem(window_sem);

				QuitIfEmpty();
			}
		break;
		
		default:
			BApplication::MessageReceived(msg);
		break;
	}
}

void
BTSRasterApp::QuitIfEmpty()
{
	int numLeft;
	
	acquire_sem(window_sem);
	numLeft = window_list.CountItems();
	release_sem(window_sem);

	if (numLeft == 0) 
	{
		PostMessage(B_QUIT_REQUESTED);
	}
}


void
BTSRasterApp::HomeDirectory(BDirectory *dir)
{
	app_info info;
	char name[1024];
	
	GetAppInfo(&info);
	BFile aFile(info.ref);
	aFile.GetPath(name, 1023);
	printf("File Path: %s\n", name);
	
	aFile.GetParent(dir);
	dir->GetPath(name, 1023);
	printf("Home Directory Path: %s\n", name);
}

void
BTSRasterApp::AddonsDirectory(BDirectory *dir)
{
	char name[1024];
	
	BDirectory *homedir = new BDirectory();
	
	HomeDirectory(homedir);
	homedir->GetDirectory("add-ons",dir);

	dir->GetPath(name, 1023);
	printf("Add Ons Path: %s\n", name);
}
