
#include "rrasterview.h"
#include <assert.h>

#define	OFFSET_H	15
#define	OFFSET_V	15

/*------------------------------------------------------------*/

BTSOutlineView::BTSOutlineView(BRect re, long flags, BRect *inside) 
	: BView(re, "", 0, flags)
{
	fInFrame = *inside;
}

/*------------------------------------------------------------*/

void	
BTSOutlineView::Draw(BRect update_rect)
{
	BRect	bound;
	BRect	tmp;

	bound = Bounds();

	tmp = fInFrame;
	tmp.InsetBy(-1, -1);
	SetHighColor(0, 0, 0);
	StrokeRect(tmp);	
	MovePenTo(BPoint(tmp.right + 1, tmp.top + 1));
	StrokeLine(BPoint(tmp.right + 1, tmp.bottom + 1));
	StrokeLine(BPoint(tmp.left + 1, tmp.bottom + 1));
}



/*------------------------------------------------------------*/


BTSRasterView::BTSRasterView(BRect re, long flags, GfxImage *aRaster) 
	: BView(re, "", 0, flags)
{
	BRect		bitmap_r;
	screen_info	info;

	get_screen_info(&info);

	fLastLoaded = aRaster;
	fHeight = aRaster->height;
	fWidth = aRaster->width;
	image = 0;

	selection.Set( -1000, -1000, -1000, -1000);

	printf("BTSRasterView:: Type: %d\n", aRaster->type);
	bitmap_r.Set(0, 0, fWidth - 1, fHeight - 1);
	the_off = new BBitmap(bitmap_r, aRaster->type, TRUE);
	off_view = new BView(bitmap_r, "", B_FOLLOW_ALL, B_WILL_DRAW);
	the_off->AddChild(off_view);
}

/*------------------------------------------------------------*/

BTSRasterView::~BTSRasterView()
{
	delete the_off;
	the_off = NULL;
	//delete cur_file;
	//cur_file = NULL;
}

/*------------------------------------------------------------*/

bool	
BTSRasterView::HasSelection()
{
	if (((selection.bottom - selection.top) + (selection.right - selection.left)) < 5) 
		return 0;
	else
		return 1;
}

/*------------------------------------------------------------*/

long	
BTSRasterView::LimitWidth(long v)
{
	if (v > (fWidth - 1))
			v = (fWidth - 1);

	if (v < 0)
			v = 0;
	return(v);
}

/*------------------------------------------------------------*/

long	
BTSRasterView::LimitHeight(long v)
{
	if (v > (fHeight - 1))
			v = fHeight - 1;

	if (v < 0)
			v = 0;
	return(v);
}

/*------------------------------------------------------------*/

static inline long	
vmax(long a, long b)
{
	if (a < b)
		return(b);
	else
		return(a);
}

/*------------------------------------------------------------*/

static inline long	
vmin(long a, long b)
{
	if (a < b)
		return(a);
	else
		return(b);
}



/*------------------------------------------------------------*/

void	
BTSRasterView::clip(long *x, long *y)
{
	if (*x > (fWidth - 1))
			*x = (fWidth - 1);
	if (*x < 0)
			*x = 0;
	if (*y > (fHeight - 1))
			*y = fHeight - 1;
	if (*y < 0)
			*y = 0;
}

/*------------------------------------------------------------*/

BRect	BTSRasterView::sort_rect(BRect *aRect)
{
	BRect	tmp_rect;
	long	tmp;

	tmp_rect = *aRect;
	if (tmp_rect.bottom < tmp_rect.top) {
		tmp = tmp_rect.top;
		tmp_rect.top = tmp_rect.bottom;
		tmp_rect.bottom = tmp;
	}

	if (tmp_rect.left > tmp_rect.right) {
		tmp = tmp_rect.right;
		tmp_rect.right = tmp_rect.left;
		tmp_rect.left = tmp;
	}

	tmp_rect.top = LimitHeight(tmp_rect.top);
	tmp_rect.left = LimitWidth(tmp_rect.left);
	tmp_rect.bottom = LimitHeight(tmp_rect.bottom);
	tmp_rect.right = LimitWidth(tmp_rect.right);

	return(tmp_rect);
}


/*------------------------------------------------------------*/

void	BTSRasterView::paste_bitmap(BMessage *msg, bool sel, char auto_place,
	BPoint loc)
{
	uchar*		raw_bits;
	uchar*		tmp_raw;
	uchar		blue;
	long		length;
	long		num_colors;
	long		i;
	BRect		bounds	= msg->FindRect("bounds");
	color_space	cs		= (color_space) msg->FindLong("color_space");
	rgb_color	*image_bits	= (rgb_color*)msg->FindData("bits", B_RAW_TYPE, &length);

	_paste_bitmap(bounds, cs, length, image_bits, sel, auto_place, loc);

/*
	if (cs == B_COLOR_8_BIT) {
		_paste_bitmap(bounds, cs, length, image_bits, sel, auto_place, loc);
	} else {
		num_pixels = (bounds.IntegerWidth() + 1) * (bounds.IntegerHeight() + 1);
		length = num_pixels * 3;
		raw_bits = (uchar*)malloc(length);

		// no gamma correct just convert BGRA -> RGB
		tmp_raw = raw_bits;
		for (i = 0; i < num_colors; i++) {
			blue = image_bits[i].red;
			*tmp_raw++ = image_bits[i].blue;
			*tmp_raw++ = image_bits[i].green;
			*tmp_raw++ = blue;
		}

		_paste_bitmap(bounds, cs, length, raw_bits, sel, auto_place, loc);
		free(raw_bits);
	}
*/
}

/*------------------------------------------------------------*/
//==============
// matchPalette
//==============

static void
matchPalette(GfxImage *image)
{
	printf("matchPalette - BEGIN\n");
	fflush(stdout);
	// Go through the image palette
	// For each entry, find its equivalent in the system palette
	// stick this new value into our local palette


	unsigned char localPalette[256];
	for (int i=0; i<256; i++)
	{
		// If the palette entry is the background color, then
		// just make that transparent.  Otherwise, lookup the correct value
		if (image->Transparent && (i == image->Background))
			localPalette[i] = B_TRANSPARENT_8_BIT;
		else
			localPalette[i] = index_for_color(image->palette[i]);
	}
	
	// Now go through all the pixels in the image
	// lookup the value in the local palette
	// and substitute this new value for the current one
	long aLength = image->height*image->bytes_per_row;
	for (int ctr = 0; ctr < aLength; ctr++)
	{
		image->data[ctr] = localPalette[image->data[ctr]];
	}
}


void	
BTSRasterView::paste_bitmap(GfxImage *image, bool sel, char auto_place,
							   BPoint loc)
{
	if (!image)
		return;
		
	printf("BTSRasterView::paste_bitmap - Type: %d\n", image->type);
	
	long size = image->bytes_per_row * image->height;
	BRect bound(0,0,image->width-1, image->height-1);

	if (image->data)
	{
		if (image->type == B_COLOR_8_BIT)
			matchPalette(image);
		_paste_bitmap(bound, image->type, size, image->data, 
			sel, auto_place, loc);
	}
}

void	
BTSRasterView::paste_bitmap(BFile *a_file, bool sel, char auto_place,
							   BPoint loc)
{
	GfxImage * tmpImage = CreateRasterFromFile(a_file);
	if (!tmpImage)
		return;
		
	paste_bitmap(tmpImage, sel, auto_place, loc);
	
	//Need to dispose of image
	//DestroyRasterImage(tmpImage);
}

/*------------------------------------------------------------*/

void 
BTSRasterView::UpdateScrollBars()
{
	BScrollView*	scrollview;
	BScrollBar*		scrollbar;
	BRect			visible_extent;
	BRect			total_extent;
	BRect			bound;
	BRect			my_bounds;
	long			max;

	Window()->Lock();
	bound = Bounds();
	bound.right += 2 * OFFSET_H;
	bound.bottom += 2 * OFFSET_V;
	scrollview = (BScrollView*)Window()->FindView("scroll view");
	my_bounds = scrollview->Bounds();
	my_bounds.right -= B_V_SCROLL_BAR_WIDTH;
	my_bounds.bottom -= B_H_SCROLL_BAR_HEIGHT;

	visible_extent = bound & my_bounds;
	total_extent = bound | my_bounds;

	scrollbar = scrollview->ScrollBar(B_HORIZONTAL);
	max = bound.Width() - my_bounds.Width();
	if (max < 0)
		max = 0;
	scrollbar->SetRange(0, max);
	scrollbar->SetProportion(visible_extent.Width() / total_extent.Width());

	scrollbar = scrollview->ScrollBar(B_VERTICAL);
	max = bound.Height() - my_bounds.Height();
	if (max < 0)
		max = 0;
	scrollbar->SetRange(0, max);
	scrollbar->SetProportion(visible_extent.Height() / total_extent.Height());
	Window()->Unlock();
}

/*------------------------------------------------------------*/

static void	
set_bits_8(BBitmap *the_bitmap, const uchar *data, long length)
{
	char		*base;
	long		i;
	long		rowb = the_bitmap->BytesPerRow();
	long		actual_width;
	BRect		bnd;
	long		tmp;

	bnd = the_bitmap->Bounds();
	actual_width = (bnd.right-bnd.left) + 1;
	base = ((char *)the_bitmap->Bits());

	tmp = actual_width;

	for (i = 0; i < length; i++) 
	{	
		if (tmp == 0) 
		{
			tmp = actual_width;
			base += rowb - actual_width;
		}
		tmp--;


		*base++ = data[i];
	}
}

static void	
set_bits_32(BBitmap *the_bitmap, const uchar *data, long length)
{
	unsigned long *base;
	long		i;
	long		rowb = the_bitmap->BytesPerRow();
	long		actual_width;
	long		actual_height;
	BRect		bnd;
	long		tmp;

	bnd = the_bitmap->Bounds();
	actual_width = (bnd.right-bnd.left) + 1;
	actual_height = (bnd.bottom - bnd.top) + 1;
	base = ((ulong *)the_bitmap->Bits());

	//printf("set_bits_32 - Width:%d Height:%d\n", actual_width, actual_height);
	//printf("set_bits_32 - BytesPerRow:%d\n", rowb);
	//printf("set_bits_32 - Length:%d\n", length);
	
	long npixels = actual_width * actual_height;
	ulong *dataPtr = (ulong *)data;
	for (int i = 0; i < npixels; i++)
	{
		base[i] = dataPtr[i];
	}
	
}

static void	
set_bits_dither(BBitmap *the_bitmap, const uchar *data, long length)
{
	char		*base;
	long		i;
	uchar		a_byte;
	rgb_color	a_color;
	rgb_color	result_color;
	long		error_r = 0;
	long		error_g = 0;
	long		error_b = 0;
	long		rowb = the_bitmap->BytesPerRow();
	long		actual_width;
	BRect		bnd;
	long		tmp;

	bnd = the_bitmap->Bounds();
	actual_width = (bnd.right-bnd.left) + 1;
	base = ((char *)the_bitmap->Bits());

	length /= 3;
	tmp = actual_width;

	for (i = 0; i < length; i++) {
		
		if (tmp == 0) {
			tmp = actual_width;
			base += rowb - actual_width;
		}
		tmp--;

		error_r = (long)*data++ - error_r;

		if (error_r > 255)
			error_r = 255;
		else
		if (error_r < 0)
			error_r = 0;
		
		error_g = (long)*data++ - error_g;
		
		if (error_g > 255)
			error_g = 255;
		else
		if (error_g < 0)
			error_g = 0;

		error_b = (long)*data++ - error_b;
		
		if (error_b > 255)
			error_b = 255;
		else
		if (error_b < 0)
			error_b = 0;


		a_color.red = error_r;
		a_color.green = error_g;
		a_color.blue = error_b;

		*base++ = a_byte = index_for_color(a_color);
		result_color = system_colors()->color_list[a_byte];
		
		
		error_r = (long)result_color.red - (long)a_color.red;
		error_g = (long)result_color.green - (long)a_color.green;
		error_b = (long)result_color.blue - (long)a_color.blue;
	}
}

void 
BTSRasterView::_paste_bitmap(BRect bound, color_space cs, long length,
	const void *data, bool sel, bool auto_place, BPoint loc)
{
	BBitmap			*tmp_off;
	BRect			tmp;
	long			h;
	long			v;

	if (bound.right > 10000)
		return;
	if (bound.right < 0)
		return;
	if (bound.bottom < 1)
		return;
	if (bound.bottom > 10000)
		return;

	printf("BTSRasterView::_paste_bitmap - Image Type: %d\n", cs);
	
	if (cs > B_RGB_32_BIT)
		cs = B_COLOR_8_BIT;
	if (cs < 0)
		cs = B_COLOR_8_BIT;

	if (cs == B_COLOR_8_BIT)
		tmp_off = new BBitmap(bound, B_COLOR_8_BIT);
	else
		tmp_off = new BBitmap(bound, the_off->ColorSpace());

	if ((tmp_off->ColorSpace() == B_COLOR_8_BIT))
	{
		if (cs == B_RGB_32_BIT)
			set_bits_dither(tmp_off, (const uchar*)data, length);
		else
			set_bits_8(tmp_off, (const uchar*)data, length);
	} else
		set_bits_32(tmp_off, (unsigned char *)data, length);

	if (!auto_place) {
		h = loc.x;
		v = loc.y;
	} else {
		h = (fWidth - (bound.right - bound.left)) / 2;
		v = (fHeight - (bound.bottom - bound.top)) / 2;
	}

	off_view->Window()->Lock();
	if (modifiers() & B_SHIFT_KEY)
		off_view->SetDrawingMode(B_OP_OVER);
	if (modifiers() & B_COMMAND_KEY)
		off_view->SetDrawingMode(B_OP_MIN);
	if (modifiers() & B_CONTROL_KEY)
		off_view->SetDrawingMode(B_OP_BLEND);

	off_view->DrawBitmap(tmp_off, BPoint(h, v));
	off_view->SetDrawingMode(B_OP_COPY);
	off_view->Window()->Unlock();

	delete tmp_off;
	tmp.Set(-1000, -1000, 1000, 1000);
	Draw(tmp);

	if (sel) {
		SetDrawingMode(B_OP_INVERT);
		BRect tmpBound = bound;
		tmpBound.OffsetTo(h,v);
		StrokeRect(selection);
		StrokeRect(tmpBound);
		selection = tmpBound;
		SetDrawingMode(B_OP_COPY);
	}
}

/*------------------------------------------------------------*/

void	
BTSRasterView::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case 'PPNT':
			{
			BPoint offset;
			BPoint loc = msg->DropPoint(&offset);
			loc = ConvertFromScreen(loc) - offset;
			paste_bitmap(msg, FALSE, FALSE, loc);
			break;
			}

		case B_SIMPLE_DATA:
			{
			record_ref	theRef;
			BFile		the_file;
			BPoint		loc(0,0);

			theRef = msg->FindRef("refs");
			if (msg->Error() == B_NO_ERROR) {
				the_file.SetRef(theRef);
				if (the_file.Error() == B_NO_ERROR) {
					if (msg->WasDropped()) {
						BPoint offset;
						loc = msg->DropPoint(&offset);
						ConvertFromScreen(&loc);
						loc = loc - offset;
					}
					paste_bitmap(&the_file, FALSE, FALSE, loc);
				}
			}
			break;
			}

		case B_MESSAGE_NOT_UNDERSTOOD:
			{
			/*
			 destination didn't understand the 'PPNT' message type. The
			 only other one is a 'ref'.
			*/
			BMessage	*original;
			bool		ref_ok = FALSE;
			long		i = 0;
			original = msg->Previous();
			//ASSERT(original);
			if (original->what == 'PPNT') {
				// try sending the data in a file instead...
				BMessage	*new_msg = convert_to_ref_message(original);
				msg->SendReply(new_msg);
			}
			break;
			}
		default:
			BView::MessageReceived(msg);
			break;
	}
}

void
BTSRasterView::DisplayBitmap(GfxImage *raster)
{
	paste_bitmap(raster, FALSE, FALSE, BPoint(0,0));
}

/*------------------------------------------------------------*/

void	
BTSRasterView::SaveBitmap(BBitmap *an_off)
{
	char	*bits;
	long	tmp_long;
	BRect 	tmpBound;

	if (cur_file == NULL)					// make sure there is a file
		return;

	if (cur_file->Open(B_READ_WRITE) != B_NO_ERROR)	// open it
		return;
	cur_file->SetSize(0);					// truncate it

	tmpBound.left = 0;
	tmpBound.top = 0;
	tmpBound.right = fWidth - 1;
	tmpBound.bottom = fHeight - 1;

	cur_file->Write("BeDump\n",7);
	tmp_long = tmpBound.left;				// write the bounding rect
	cur_file->Write((char *)&tmp_long, 4);
	tmp_long = tmpBound.top;
	cur_file->Write((char *)&tmp_long, 4);
	tmp_long = tmpBound.right;
	cur_file->Write((char *)&tmp_long, 4);
	tmp_long = tmpBound.bottom;
	cur_file->Write((char *)&tmp_long, 4);
	
	color_space tc = an_off->ColorSpace();	// write the color space
	tmp_long = tc;
	cur_file->Write((char *)&tmp_long, 4);
	long bl = an_off->BitsLength();
	cur_file->Write((char *)&(bl), 4);
	bits = (char*)an_off->Bits();
	cur_file->Write((char *)bits, an_off->BitsLength());	// and bitmap
	cur_file->Close();
	cur_file->SetTypeAndApp('GRIM', 'GRVW');
}

/*------------------------------------------------------------*/

void	
BTSRasterView::SaveBitmap24(BBitmap *an_off)
{
	uchar		*bits;
	BVolume		volume;
	BDirectory	dir;
	BFile		raw_file;
	color_map	*the_map;
	long		bl;
	uchar		tmp_array[90];
	char		name[B_FILE_NAME_LENGTH];
	long		i;
	long		x;
	uchar		tmp_buf[3];

	the_map = system_colors();

	volume = boot_volume();
	volume.GetRootDirectory(&dir);
	
	strcpy(name, "screen.raw");
	i = 1;
	while (dir.Contains(name))
		sprintf(name, "screen.raw.%d", ++i);

	if (dir.Create(name, &raw_file) != B_NO_ERROR)
		return;
	
	raw_file.Open(B_READ_WRITE);
	
	bl = an_off->BitsLength();
	bits = (uchar*)an_off->Bits();

	if (an_off->ColorSpace() == B_COLOR_8_BIT) {
		while(bl > 30) {
			for (i = 0; i < 30; i++) {
				memcpy(&(tmp_array[i*3]), &(the_map->color_list[*bits++]), 3);
			}
			raw_file.Write((char *)&tmp_array, 90);
			bl -= 30;
		}
		while(bl > 0) {
			raw_file.Write((char *)&(the_map->color_list[*bits++]), 3);
			bl--;
		}
	} else {
		if (an_off->ColorSpace() == B_RGB_32_BIT) {
			while(bl > 120) {
				for (i = 0; i < 90; i += 3) {
					tmp_array[i + 2] = *bits++;
					tmp_array[i + 1] = *bits++;
					tmp_array[i] = *bits++;
					bits++;
				}

				raw_file.Write((char*)&tmp_array, 90);
				bl -= 120;
			}

			while(bl > 0) {
				tmp_array[2] = *bits++;
				tmp_array[1] = *bits++;
				tmp_array[0] = *bits++;
				bits++;
				bl -= 4;
				raw_file.Write((char*)&tmp_array, 3);
			}
		}
	}

	raw_file.Close();
}

/*------------------------------------------------------------*/

void	
BTSRasterView::Save()
{
	SaveBitmap(the_off);
}

/*------------------------------------------------------------*/

void	
BTSRasterView::Save24()
{
	SaveBitmap24(the_off);
}

/*------------------------------------------------------------*/

BMessage *
BTSRasterView::convert_to_ref_message(BMessage *msg)
{
	BDirectory	root_dir;
	BDirectory	tmp_dir;
	BFile		*a_file;
	BVolume		tmp_volume;

	tmp_volume = boot_volume();
	tmp_volume.GetRootDirectory(&root_dir);
	if (root_dir.GetDirectory("tmp", &tmp_dir) != B_NO_ERROR)
		root_dir.Create("tmp", &tmp_dir);
	//ASSERT(root_dir.Error() == B_NO_ERROR);

	a_file = write_bitmap(msg, "clip", &tmp_dir);
	msg = new BMessage(B_SIMPLE_DATA /* B_REFS_RECEIVED */);
	msg->AddRef("refs", a_file->Record()->Ref());
	delete a_file;

	return msg;
}

/*------------------------------------------------------------*/

BMessage	*
BTSRasterView::create_clip_message(BBitmap *bm)
{
	BMessage *msg = new BMessage('PPNT');

	msg->AddRect("bounds", bm->Bounds());
	msg->AddLong("color_space", bm->ColorSpace());
	msg->AddLong("bits_length", bm->BitsLength());
	msg->AddData("bits", B_RAW_TYPE, bm->Bits(), bm->BitsLength());
	
	return msg;
}

/*------------------------------------------------------------*/

BFile	*
BTSRasterView::write_bitmap(BMessage *msg, char *fname, BDirectory *dir)
{
	char		*bits;
	long		bl;
	long		temp;
	BFile		*a_file;
	BRect 		tmpBound;
	BRect 		bnd;
	color_space	tc;	
	long		e0;

	a_file = new BFile();
	
	e0 = dir->Create(fname, a_file);
	if (dir->Error()) {
		e0 = dir->GetFile(fname, a_file);
		e0 = a_file->Open(B_READ_WRITE);
	}

	long	tmp_long;

	bnd = msg->FindRect("bounds");
	tmpBound = bnd;
	a_file->Open(B_READ_WRITE);	
	tmp_long = tmpBound.left;
	a_file->Write((char *)&tmp_long, 4);
	tmp_long = tmpBound.top;
	a_file->Write((char *)&tmp_long, 4);
	tmp_long = tmpBound.right;
	a_file->Write((char *)&tmp_long, 4);
	tmp_long = tmpBound.bottom;
	a_file->Write((char *)&tmp_long, 4);

	tc = (color_space) msg->FindLong("color_space");
	temp = tc;
	a_file->Write((char *)&(temp), 4);
	bl = msg->FindLong("bits_length");
	a_file->Write((char *)&(bl), 4);
	bits = (char *) msg->FindData("bits", B_RAW_TYPE, &temp);
	a_file->Write((char *)bits, bl);
	a_file->Close();
	a_file->SetTypeAndApp('BITM', 'PPNT');
	return(a_file);
}

/*------------------------------------------------------------*/

void	
BTSRasterView::drag_selection(long h, long v, BPoint mouseDown)
{
	BBitmap			*selection_bm;
	BRect			selection_bound;
	BRect			new_dst;
	BRect			old_dst;
	long			h0, v0;
	long			h1, v1;
	ulong			buttons;
	long			dh, dv;
	BView			*tmp_view;
	BRect			vert, horiz;
	BRect			view_bound;
	BMessage		*msg;

	clip(&h, &v);

	dv = dh = 0;

	view_bound = Bounds();

	selection_bound.top = 0;
	selection_bound.bottom = selection.bottom - selection.top;
	selection_bound.left = 0;
	selection_bound.right = selection.right - selection.left;

	selection_bm = new BBitmap(selection_bound, the_off->ColorSpace(), TRUE);
	selection_bm->Lock();
	selection_bm->AddChild(tmp_view = new BView(selection_bound, "", B_FOLLOW_ALL, B_WILL_DRAW));
	tmp_view->DrawBitmap(the_off, selection, selection_bound);
	selection_bm->Unlock();

	msg = create_clip_message(selection_bm);

	dh = mouseDown.x;
	dv = mouseDown.y;
	dv -= selection.top;
	dh -= selection.left;

	//PRINT(("dragging messaage (%.4s)\n", (char *) &(msg->what)));
	DragMessage(msg, selection_bm, BPoint(dh, dv));
}

/*------------------------------------------------------------*/

void		
BTSRasterView::RemoveSelection()
{
	if (HasSelection()) 
	{
		SetDrawingMode(B_OP_INVERT);
		StrokeRect(selection);
		SetDrawingMode(B_OP_COPY);
		selection.Set(-1000, -1000, -1000, -1000);
	}
}




/*------------------------------------------------------------*/

void	
BTSRasterView::change_selection(long h, long v)
{
	ulong	buttons;
	long	h0;
	long	v0;
	BRect	new_select;
	BRect	old_select;
	BRect	tmp_rect;
	
	clip(&h, &v);
	new_select.top = v;
	new_select.left = h;
	old_select = selection;

	SetDrawingMode(B_OP_INVERT);

	do {
		BPoint where;
		GetMouse(&where, &buttons);
		h0 = where.x;
		v0 = where.y;
		clip(&h0, &v0);
		new_select.right = h0;
		new_select.bottom = v0;

		if ((old_select.top != new_select.top) || 
		    (old_select.bottom != new_select.bottom) ||
		    (old_select.right != new_select.right) ||
		    (old_select.left != new_select.left)) {
		
			tmp_rect = sort_rect(&new_select);
			StrokeRect(tmp_rect);

			tmp_rect = sort_rect(&old_select);
			StrokeRect(tmp_rect);

			old_select = new_select;
			Flush();
		}

		snooze(20000.0);
	} while(buttons);

	selection = sort_rect(&new_select);
	if (!HasSelection()) {
		StrokeRect(selection);
		selection.Set(-1000, -1000, -1000, -1000);
	} 
	SetDrawingMode(B_OP_COPY);
}



/*------------------------------------------------------------*/

void	
BTSRasterView::MouseDown(BPoint where)
{
	long x = where.x;
	long y = where.y;
	BRect		new_select;
	BRect		old_select;
	BRect		tmp_rect;

	if ((HasSelection()) && selection.Contains(BPoint(x, y))) 
	{
		drag_selection(x, y, where);
		return;
	}

	// Create a new selection rectangle
	change_selection(x, y);
}

/*------------------------------------------------------------*/
//=================================================
// Method: ClearImage
//
//=================================================

void	
BTSRasterView::ClearImage()
{
	char	*tmp;
	long	i;
	long	j;

	tmp = (char *)malloc(fWidth);

	for (j = 0; j < fWidth; j++) 
	{
		*(tmp + j) = 16;
	}

	for (i = 0; i < (fHeight); i++) 
	{
		the_off->SetBits((char*)tmp, fWidth, fWidth * i,
			B_COLOR_8_BIT);
	}

	free((char *)tmp);
}

/*------------------------------------------------------------*/

void	
BTSRasterView::SetImage()
{
	char	*tmp1;
	char	*tmp2;
	long	i;
	long	j;
	uchar	*translator;
	uchar	*image1;

/*
	translator = (uchar *)&(control_window->translator);
	if (translator == 0) {
		//printf("no trans\n");
		return;
	}
	if (image == 0) {
		//printf("no image\n");
		return;
	}

	tmp1 = (char *)malloc(fWidth * 2);

	image1 = image;

	if (has_selection()) {
		for (i = selection.top + 1; i < selection.bottom; i++) {
			image1 = image + (i * fWidth) + (int)selection.left + 1;
			tmp2 = tmp1;
			for (j = 0; j < (selection.right - selection.left); j++) {
				*tmp2++ = (translator[*image1++ >> 1] >> 3);
			}
			the_off->SetBits((char*)tmp1, selection.right - selection.left,
				i * fWidth + selection.left + 1, B_COLOR_8_BIT); //???
		}
	}
	else
	for (i = 0; i < (fHeight / 2); i++) {
		tmp2 = tmp1;
		for (j = 0; j < fWidth; j++) {
			*tmp2++ = (translator[*image1++ >> 1] >> 3);
			*tmp2++ = (translator[*image1++ >> 1] >> 3);
		}

		the_off->SetBits((char*)tmp1, (fWidth * 2), (fWidth * 2) * i,
				 B_COLOR_8_BIT); //???
	}

	free((char *)tmp1);
*/
}

/*------------------------------------------------------------*/

static ulong	
matrix   (long p00, long p01, long p02,
		  long p10, long p11, long p12,
		  long p20, long p21, long p22)
{
	long	result;

	result = (p11 * 12) - p00 - p01 - p02 - p10 - p11 - p12 - p20 - p21 - p22;
	result = result / 4;
	return(result);
}



/*------------------------------------------------------------*/

void	
BTSRasterView::Draw(BRect update_rect)
{
	DrawBitmap(the_off, BPoint(0, 0));

	SetDrawingMode(B_OP_INVERT);
	StrokeRect(selection);
	SetDrawingMode(B_OP_COPY);
}

